@@ -35,7 +35,7 @@
 #include <signal.h>
 
 #include "glib.h"
-#include "gthreadinit.h"
+#include "gthreadprivate.h"
 #include "galias.h"
 
 #define MEM_PROFILE_TABLE_SIZE 4096
@@ -318,7 +318,8 @@ profiler_log (ProfilerJob job,
   g_mutex_lock (gmem_profile_mutex);
   if (!profile_data)
     {
-      profile_data = standard_malloc ((MEM_PROFILE_TABLE_SIZE + 1) * 8 * sizeof (profile_data[0]));
+      profile_data = standard_calloc ((MEM_PROFILE_TABLE_SIZE + 1) * 8, 
+                                      sizeof (profile_data[0]));
       if (!profile_data)	/* memory system kiddin' me, eh? */
 	{
 	  g_mutex_unlock (gmem_profile_mutex);
@@ -688,7 +689,7 @@ g_mem_init_nomessage (void)
 {
   gchar buffer[1024];
   const gchar *val;
-  static const GDebugKey keys[] = {
+  const GDebugKey keys[] = {
     { "gc-friendly", 1 },
   };
   gint flags;
