@@ -202,18 +202,19 @@ g_once_impl (GOnce       *once,
 gboolean
 g_once_init_enter_impl (volatile gsize *value_location)
 {
-  gboolean need_init;
+  gboolean need_init = FALSE;
   g_mutex_lock (g_once_mutex);
-  if (!g_once_init_list || !g_slist_find (g_once_init_list, (void*) value_location))
+  if (g_atomic_pointer_get ((void**) value_location) == 0)
     {
-      g_once_init_list = g_slist_prepend (g_once_init_list, (void*) value_location);
-      need_init = TRUE;
-    }
-  else
-    {
-      while (g_slist_find (g_once_init_list, (void*) value_location))
-        g_cond_wait (g_once_cond, g_once_mutex);
-      need_init = FALSE;
+      if (!g_slist_find (g_once_init_list, (void*) value_location))
+        {
+          need_init = TRUE;
+          g_once_init_list = g_slist_prepend (g_once_init_list, (void*) value_location);
+        }
+      else
+        do
+          g_cond_wait (g_once_cond, g_once_mutex);
+        while (g_slist_find (g_once_init_list, (void*) value_location));
     }
   g_mutex_unlock (g_once_mutex);
   return need_init;
@@ -223,11 +224,11 @@ void
 g_once_init_leave (volatile gsize *value_location,
                    gsize           initialization_value)
 {
-  g_return_if_fail (g_atomic_pointer_get (value_location) == 0);
+  g_return_if_fail (g_atomic_pointer_get ((void**) value_location) == 0);
   g_return_if_fail (initialization_value != 0);
   g_return_if_fail (g_once_init_list != NULL);
 
-  g_atomic_pointer_set (value_location, initialization_value);
+  g_atomic_pointer_set ((void**) value_location, (void*) initialization_value);
   g_mutex_lock (g_once_mutex);
   g_once_init_list = g_slist_remove (g_once_init_list, (void*) value_location);
   g_cond_broadcast (g_once_cond);
@@ -255,7 +256,7 @@ g_static_mutex_get_mutex_impl (GMutex** mutex)
   g_mutex_lock (g_once_mutex);
 
   if (!(*mutex))
-    g_atomic_pointer_set (mutex, g_mutex_new());
+    g_atomic_pointer_set ((void**) mutex, g_mutex_new());
 
   g_mutex_unlock (g_once_mutex);
 
@@ -579,7 +580,7 @@ g_thread_cleanup (gpointer data)
 
 	  /* Just to make sure, this isn't used any more */
 	  g_system_thread_assign (thread->system_thread, zero_thread);
-	  g_free (thread);
+          g_free (thread);
 	}
     }
 }
@@ -662,8 +663,11 @@ g_thread_create_full (GThreadFunc 		 func,
   G_THREAD_UF (thread_create, (g_thread_create_proxy, result,
 			       stack_size, joinable, bound, priority,
 			       &result->system_thread, &local_error));
-  result->next = g_thread_all_threads;
-  g_thread_all_threads = result;
+  if (!local_error)
+    {
+      result->next = g_thread_all_threads;
+      g_thread_all_threads = result;
+    }
   G_UNLOCK (g_thread);
 
   if (local_error)
